#!/bin/sh
#
#  PlanetsCentral - Run Master
#
#  First parameter is game directory, "games/$gid"
#
#  Exit status:
#    1 - master preparation or run failed
#    2 - master postprocessing failed
#    3 - general error
#

if test -z "$1"; then
  echo "usage: $0 <gamedir>"
  exit 3
fi
gamedir="$1"

# Does directory exist?
for i in "" "/in" "/out" "/data" "/backup"; do
  if test -d "$gamedir$i"; then
    :
  else
    echo "Error: '$gamedir$i' does not exist"
    exit 1
  fi
done

# Log file
logfile="$gamedir/runmaster.log"
echo "Log file for '$0 $gamedir' on `date`" >"$logfile"

# Read Configuration
. "$gamedir/c2host.ini"
spec_mandatory="amaster.src beamspec.dat engspec.dat hullspec.dat pconfig.src planet.nm race.nm storm.nm torpspec.dat truehull.dat"
spec_optional="shiplist.txt hullfunc.txt xyplan.dat"

# Check configuration
if test -x "$game_master_path/$game_master_program"; then
  :
else
  echo "Error: master program '$game_master_path/$game_master_program' does not exist."
  exit 1
fi

# Install all game files. Priority order is:
# - game
# - shiplist
# - master
# - host
# - default
# Mandatory files first:
echo "Copying spec files..." >>$logfile
for f in $spec_mandatory; do
  if test -e "$gamedir/data/$f"; then
    : # ok
  else
    ok=false
    for src in "$game_sl_path/$f" "$game_master_path/$f" "$game_host_path/$f" "defaults/$f"; do
      if cp "$src" "$gamedir/data/$f" 2>/dev/null; then
        ok=true
        echo "  Using $src" >>"$logfile"
        break
      fi
    done
    if $ok; then
      : # ok
    else
      echo "Error: unable to find file '$f'"
      exit 1
    fi
  fi
done
for f in $spec_optional; do
  if test -e "$gamedir/data/$f"; then
    : # ok
  else
    for src in "$game_sl_path/$f" "$game_master_path/$f" "$game_host_path/$f" "defaults/$f"; do
      if cp "$src" "$gamedir/data/$f" 2>/dev/null; then
        echo "  Using $src" >>"$logfile"
        break
      fi
    done
  fi
done

# Install all configuration fragments
echo "Updating configuration..." >>$logfile

# Find tools
toolpaths=""
if test -n "$game_tools"; then
  for i in $game_tools; do
    eval "toolpaths=\"\$toolpaths \$game_tool_${i}_path\""
  done
fi

# Find fragments
afrags=""
pfrags=""
sfrags=""
# mfrags=""
for src in $game_sl_path $game_master_path $game_host_path defaults $toolpaths; do
  # Priority order is tools, shiplist, master, host, default, so we must build options
  # for updateconfig in reverse order!
  f="$src/amaster.src.frag"
  if test -r "$f"; then
    echo "  Using $f for amaster.src" >>"$logfile"
    afrags="--file=$f $afrags"
  fi
  f="$src/pconfig.src.frag"
  if test -r "$f"; then
    echo "  Using $f for pconfig.src" >>"$logfile"
    pfrags="--file=$f $pfrags"
  fi
  f="$src/shiplist.txt.frag"
  if test -r "$f"; then
    echo "  Using $f for shiplist.txt" >>"$logfile"
    sfrags="--file=$f $sfrags"
  fi
  # f="$src/mission.ini"
  # if test -r "$f"; then
  #   echo "  Using $f for mission.ini" >>"$logfile"
  #   mfrags="$mfrags $f"
  # fi
done

# Update PHost configuration
perl bin/updateconfig.pl "$gamedir/data/pconfig.src" $pfrags "PHOST.GameName=$game_name" || exit 1
perl bin/updateconfig.pl "$gamedir/data/shiplist.txt" $sfrags                            || exit 1

# Update AMaster configuration. Some PHost options are mirrored in amaster.src.
tmpfrag="$gamedir/tmpamaster.frag"
(
  echo '%AMASTER'
  grep -wiE 'PlayerRace|MapTruehullByPlayerRace|AllowWraparoundMap|WraparoundRectangle|CrystalsPreferDeserts|CrystalSinTempBehavior|ClimateLimitsPopulation|MaxColTempSlope' <"$gamedir/data/pconfig.src"
) > "$tmpfrag"
nlines=$(wc -l <"$tmpfrag")
echo "  Updating amaster.src with" $nlines "lines from pconfig.src" >>"$logfile"
perl bin/updateconfig.pl "$gamedir/data/amaster.src" $afrags --file="$tmpfrag" "AMASTER.RaceIsPlaying=$game_slots" || exit 1
rm -f "$tmpfrag"

# Generate mission.ini
# if test -n "$mfrags"; then
#   for i in $mfrags; do
#     cat "$i"
#     echo
#   done > $gamedir/data/mission.ini
# fi

# Initialize tools
for src in $toolpaths; do
  f="$src/c2master.sh"
  if test -r "$f"; then
    echo "  Executing $f..." >>"$logfile"
    set -- "$gamedir/data" "$src"
    . "$f" >>$logfile || exit 1
  fi
done

# Pre-master backup
echo "Pre-Master backup..." >>$logfile
( cd "$gamedir/data" && tar czf "../backup/premaster.tgz" . ) || exit 1

# Prepare
if test "$game_host_kind" = "phost4"; then
  PHOST_VERSION=4
else
  PHOST_VERSION=3
fi
export PHOST_VERSION

# Run master
if test "$game_settings_masterHasRun" = "1"; then
  echo "Not running master because masterHasRun is set." >>$logfile
else
  echo "Running master ($game_master_path/$game_master_program)..." >>$logfile
  "$game_master_path/$game_master_program" $game_master_options "$gamedir/data" "$game_master_path" || exit 1
fi

# No post-master backup; that would be the the same as the next pre-host backup
exit 0
